iov_iter is implemented on bvec itererator helpers, so it is safe to pass
multi-page bvec to it, and this way is much more efficient than passing one
page in each bvec.

Reviewed-by: Christoph Hellwig <h...@lst.de>
Reviewed-by: Omar Sandoval <osan...@fb.com>
Signed-off-by: Ming Lei <ming....@redhat.com>
---
 drivers/block/loop.c | 20 ++++++++++----------
 1 file changed, 10 insertions(+), 10 deletions(-)

diff --git a/drivers/block/loop.c b/drivers/block/loop.c
index cf5538942834..168a151aba49 100644
--- a/drivers/block/loop.c
+++ b/drivers/block/loop.c
@@ -511,21 +511,22 @@ static int lo_rw_aio(struct loop_device *lo, struct 
loop_cmd *cmd,
                     loff_t pos, bool rw)
 {
        struct iov_iter iter;
+       struct req_iterator rq_iter;
        struct bio_vec *bvec;
        struct request *rq = blk_mq_rq_from_pdu(cmd);
        struct bio *bio = rq->bio;
        struct file *file = lo->lo_backing_file;
+       struct bio_vec tmp;
        unsigned int offset;
-       int segments = 0;
+       int nr_bvec = 0;
        int ret;
 
+       rq_for_each_mp_bvec(tmp, rq, rq_iter)
+               nr_bvec++;
+
        if (rq->bio != rq->biotail) {
-               struct req_iterator iter;
-               struct bio_vec tmp;
 
-               __rq_for_each_bio(bio, rq)
-                       segments += bio_segments(bio);
-               bvec = kmalloc_array(segments, sizeof(struct bio_vec),
+               bvec = kmalloc_array(nr_bvec, sizeof(struct bio_vec),
                                     GFP_NOIO);
                if (!bvec)
                        return -EIO;
@@ -534,10 +535,10 @@ static int lo_rw_aio(struct loop_device *lo, struct 
loop_cmd *cmd,
                /*
                 * The bios of the request may be started from the middle of
                 * the 'bvec' because of bio splitting, so we can't directly
                 * copy bio->bi_iov_vec to new bvec. The rq_for_each_segment
                 * API will take care of all details for us.
                 */
-               rq_for_each_segment(tmp, rq, iter) {
+               rq_for_each_mp_bvec(tmp, rq, rq_iter) {
                        *bvec = tmp;
                        bvec++;
                }
@@ -551,11 +552,10 @@ static int lo_rw_aio(struct loop_device *lo, struct 
loop_cmd *cmd,
                 */
                offset = bio->bi_iter.bi_bvec_done;
                bvec = __bvec_iter_bvec(bio->bi_io_vec, bio->bi_iter);
-               segments = bio_segments(bio);
        }
        atomic_set(&cmd->ref, 2);
 
-       iov_iter_bvec(&iter, rw, bvec, segments, blk_rq_bytes(rq));
+       iov_iter_bvec(&iter, rw, bvec, nr_bvec, blk_rq_bytes(rq));
        iter.iov_offset = offset;
 
        cmd->iocb.ki_pos = pos;
@@ -534,10 +535,10 @@ static int lo_rw_aio(struct loop_device *lo, struct 
                 * The bios of the request may be started from the middle of
                 * the 'bvec' because of bio splitting, so we can't directly
-                * copy bio->bi_iov_vec to new bvec. The rq_for_each_segment
+                * copy bio->bi_iov_vec to new bvec. The rq_for_each_mp_bvec
                 * API will take care of all details for us.
                 */
-- 
2.9.5

Reply via email to